#include <windows.h>
#include "NetworkRemocon.h"


class CSendStringInfo {
public:
	CNetworkRemocon *m_pRemocon;
	const char *m_pBuffer;
	int m_Length;
	CSendStringInfo(CNetworkRemocon *pRemocon,const char *pBuffer,int Length) {
		m_pRemocon=pRemocon;
		m_pBuffer=pBuffer;
		m_Length=Length;
	}
};


class CChannelMap {
	struct Map {
		int From;
		int To;
	};
public:
	CChannelMap();
	~CChannelMap();
	bool Add(int Form,int To);
};




CNetworkRemocon::CNetworkRemocon()
{
	m_fInitialized=false;
	m_hThread=NULL;
	m_Socket=INVALID_SOCKET;
	m_fConnected=false;
}


CNetworkRemocon::~CNetworkRemocon()
{
	if (m_fInitialized) {
		if (m_hThread!=NULL) {
			if (WaitForSingleObject(m_hThread,10000)==WAIT_TIMEOUT)
				TerminateThread(m_hThread,FALSE);
			CloseHandle(m_hThread);
		}
		if (m_Socket!=INVALID_SOCKET) {
			shutdown(m_Socket,1);
			closesocket(m_Socket);
		}
		WSACleanup();
	}
}


bool CNetworkRemocon::Init(LPCSTR pszAddress,WORD Port)
{
	int Err;

	if (!m_fInitialized) {
		Err=WSAStartup(MAKEWORD(2,0),&m_WSAData);
		if (Err!=0)
			return false;
		m_fInitialized=true;
	}
	m_Address=inet_addr(pszAddress);
	m_Port=Port;
	m_Socket=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
	if (m_Socket==INVALID_SOCKET)
		return false;
	return true;
}


DWORD CNetworkRemocon::SendProc(LPVOID pParam)
{
	CSendStringInfo *pInfo=(CSendStringInfo*)pParam;
	int Result;
	char Buffer[256];

	if (pInfo->m_pRemocon->m_Socket==INVALID_SOCKET) {
		pInfo->m_pRemocon->m_Socket=socket(AF_INET,SOCK_STREAM,IPPROTO_TCP);
		if (pInfo->m_pRemocon->m_Socket==INVALID_SOCKET)
			return FALSE;
	}
	if (!pInfo->m_pRemocon->m_fConnected) {
		struct sockaddr_in sockaddri;

		sockaddri.sin_family=AF_INET;
		sockaddri.sin_port=htons(pInfo->m_pRemocon->m_Port);
		sockaddri.sin_addr.S_un.S_addr=pInfo->m_pRemocon->m_Address;
		ZeroMemory(sockaddri.sin_zero,sizeof(sockaddri.sin_zero));
		Result=connect(pInfo->m_pRemocon->m_Socket,
							(struct sockaddr*)&sockaddri,sizeof(sockaddr_in));
		if (Result!=0) {
			delete pInfo;
			return FALSE;
		}
		pInfo->m_pRemocon->m_fConnected=true;
	}
	Result=send(pInfo->m_pRemocon->m_Socket,pInfo->m_pBuffer,pInfo->m_Length,0);
	if (Result==SOCKET_ERROR) {
		closesocket(pInfo->m_pRemocon->m_Socket);
		pInfo->m_pRemocon->m_Socket=INVALID_SOCKET;
		pInfo->m_pRemocon->m_fConnected=false;
		return FALSE;
	}
	Result=recv(pInfo->m_pRemocon->m_Socket,Buffer,sizeof(Buffer),0);
	delete pInfo;
	if (Result==SOCKET_ERROR || Result==0) {
		closesocket(pInfo->m_pRemocon->m_Socket);
		pInfo->m_pRemocon->m_Socket=INVALID_SOCKET;
		pInfo->m_pRemocon->m_fConnected=false;
		return Result==SOCKET_ERROR;
	}
	return TRUE;
}


bool CNetworkRemocon::Send(const char *pBuffer,int Length)
{
	DWORD ThreadID;
	CSendStringInfo *pInfo;

	if (!m_fInitialized)
		return false;
	if (m_hThread!=NULL) {
		if (WaitForSingleObject(m_hThread,0)==WAIT_TIMEOUT)
			return false;
		CloseHandle(m_hThread);
	}
	pInfo=new CSendStringInfo(this,pBuffer,Length);
	m_hThread=CreateThread(NULL,0,SendProc,pInfo,0,&ThreadID);
	if (m_hThread==NULL) {
		delete pInfo;
		return false;
	}
	return true;
}


bool CNetworkRemocon::SetChannel(int ChannelNo)
{
	char szText[16];
	int Length;

	if (m_ChannelList.NumChannels()>0) {
		int i;

		for (i=0;i<m_ChannelList.NumChannels();i++) {
			if (m_ChannelList.GetChannelNo(i)==ChannelNo+1) {
				Length=wsprintfA(szText,"SetCh:%d",i);
				return Send(szText,Length);
			}
		}
	} else {
		Length=wsprintfA(szText,"SetCh:%d",ChannelNo);
		return Send(szText,Length);
	}
	return false;
}


bool CNetworkRemocon::LoadChannelText(LPCTSTR pszFileName,
											const CChannelList *pChannelList)
{
	HANDLE hFile;
	DWORD FileSize,Read;
	char *pszText,*p;

	hFile=CreateFile(pszFileName,GENERIC_READ,FILE_SHARE_READ,NULL,
														OPEN_EXISTING,0,NULL);
	if (hFile==INVALID_HANDLE_VALUE)
		return false;
	FileSize=GetFileSize(hFile,NULL);
	if (FileSize==INVALID_FILE_SIZE || FileSize==0) {
		CloseHandle(hFile);
		return false;
	}
	pszText=new char[FileSize+1];
	if (!ReadFile(hFile,pszText,FileSize,&Read,NULL) || Read!=FileSize) {
		CloseHandle(hFile);
		return false;
	}
	pszText[FileSize]='\0';
	CloseHandle(hFile);
	p=pszText;
	while (*p!='\0') {
		int Space,Channel;

		while (*p=='\r' || *p=='\n')
			p++;
		if (*p==';') {	// Comment
			while (*p!='\r' && *p!='\n' && *p!='\0')
				p++;
			continue;
		}
		// Skip channel name
		while (*p!='\t' && *p!='\0')
			p++;
		if (*p=='\0')
			break;
		p++;
		Space=0;
		while (*p>='0' && *p<='9') {
			Space=Space*10+(*p-'0');
			p++;
		}
		if (*p!='\t')
			break;
		p++;
		Channel=0;
		while (*p>='0' && *p<='9') {
			Channel=Channel*10+(*p-'0');
			p++;
		}
		Channel+=FIRST_CHANNEL;
		for (int i=0;i<pChannelList->NumChannels();i++) {
			if (pChannelList->GetChannel(i)==Channel) {
				m_ChannelList.AddChannel(Space,Channel,
									pChannelList->GetChannelNo(i),TEXT(""));
				break;
			}
		}
		while (*p!='\r' && *p!='\n' && *p!='\0')
			p++;
	}
	delete [] pszText;
	return true;
}
